from __future__ import print_function # Python 2/3 compatibility
import os
import boto3
import json
import decimal
from boto3.dynamodb.conditions import Key, Attr
#import logging
#import sys
import requests
import adal
import calendar, datetime, time


# Helper class to convert a DynamoDB item to JSON.
class DecimalEncoder(json.JSONEncoder):
    def default(self, o):
        if isinstance(o, decimal.Decimal):
            if o % 1 > 0:
                return float(o)
            else:
                return int(o)
        return super(DecimalEncoder, self).default(o)

def adAuthFlow ():

	authority_url = (os.environ['authorityHostUrl'] + '/' +
					 os.environ['tenant'])
	RESOURCE = os.environ['resource']
	dynamodb = boto3.resource('dynamodb')
	table = dynamodb.Table(os.environ['DB_SMRIOTAD_AT'])
	a_timestamp = round(time.time())

	token_validity = table.query(
					KeyConditionExpression=Key('record_type').eq('accessToken'),
					ScanIndexForward=False,
					Limit=1
					)
	for item in token_validity['Items']:
		ttimestamp = item['timestamp']
		taccessToken = item['value']
	
	if (a_timestamp - ttimestamp) > 3600:
	
		context = adal.AuthenticationContext(
			authority_url, validate_authority=os.environ['tenant'] != 'adfs',
			)

		token = context.acquire_token_with_client_credentials(
			RESOURCE,
			os.environ['clientId'],
			os.environ['clientSecret'])
			
		table.put_item(
			Item={
				'record_type' : 'accessToken',
				'value' : token,
				'timestamp' : a_timestamp
			}
		)
	else:
		token = taccessToken

#	print('Here is the token:')
#	print(json.dumps(token, indent=2))

	return(token['accessToken'])

def check_room_reservation(roomname,a_timestamp,timer):

	from datetime import datetime
	
	temp_token = adAuthFlow()
	
	a_timestamp -= timer
	a_timestamp_utc = datetime.fromtimestamp(a_timestamp)
	a_timestamp_iso = a_timestamp_utc.strftime("%Y-%m-%dT%H:%M:%S")
	endpoint = os.environ['resource'] + '/' + 'v1.0' + "/users/" + roomname + '@' + os.environ['aad_domain'] + '/events?$filter=start/dateTime lt \'' + a_timestamp_iso + '\'&$select=organizer,id,start'
	http_headers = {'Authorization': 'Bearer ' + temp_token,
					'Content-Type': 'application/json'}

	graph_data = requests.get(endpoint, headers=http_headers, stream=False)
	room_reservation = graph_data.json()
	
	return room_reservation

def release_room (roomname,eventid):

	temp_token = adAuthFlow()
	endpoint = os.environ['resource'] + '/' + 'v1.0' + "/users/" + roomname + '@' + os.environ['aad_domain'] + "/events/" + eventid
	http_headers = {'Authorization': 'Bearer ' + temp_token,
					'Content-Type': 'application/json'}

	resp = requests.delete(endpoint, headers=http_headers)	

#	print(resp.status_code)
	return resp

def convtime_epoch(utcdt):

	from datetime import datetime
	format = '%Y-%m-%dT%H:%M:%S.%f'
	datestring = utcdt
	d = datetime.datetime.strptime(datestring, format)
	c_timestamp = datetime.timestamp(d)
	
	return c_timestamp
	
def lambda_handler(event, context):
    
	dynamodb = boto3.resource('dynamodb')
	state_table = dynamodb.Table(os.environ['DB_TABLE_STATE'])
	loc_table = dynamodb.Table(os.environ['DB_TABLE_LOCATION'])
	timer = event['occupancy_timer']
	
	list_rooms = loc_table.scan()
	room_results = []

	for lrooms in list_rooms['Items']:
    
		roomid = os.environ['topic_state'] + "/" + lrooms['topic'][20:]
		a_timestamp = round(time.time())
		
		resp_rooms = state_table.query(
				KeyConditionExpression=Key('topic').eq(roomid),
				ScanIndexForward=False,
				Limit=1
				)

		if not resp_rooms['Items']:
#			print("No room activity for ",lrooms['topic'][20:],", it is available")
			room_state = 1
		else:
			for i in resp_rooms['Items']:
				t_difference_sec = (int(a_timestamp) - int(i['timestamp']) / 1000)
				room_state = t_difference_sec > timer

		if room_state:
			room_st = "Available"
#			print("Room:",lrooms['topic'][20:], room_st)
			room_reservation = check_room_reservation(lrooms['topic'][20:],a_timestamp,timer)
			
			if room_reservation:
				for eventline in room_reservation['value']:
#					print(eventline)
					release_room(lrooms['topic'][20:],eventline['id'])
		else:
			room_st = "Occupied"
#			print("Room:",lrooms['topic'][20:], room_st)

		room_results.append({
					'Room' :   lrooms['topic'][20:],
					'State':   room_st
					})

		loc_table.update_item(
			Key={
				'topic': lrooms['topic'],
			},
			UpdateExpression='SET Occupancy = :val1',
			ExpressionAttributeValues={
				':val1': room_st
			}
		)
	return room_results