from __future__ import print_function # Python 2/3 compatibility
import os
import boto3
import json
import decimal
from boto3.dynamodb.conditions import Key, Attr
#import logging
#import sys
import requests
import adal
import time


# Helper class to convert a DynamoDB item to JSON.
class DecimalEncoder(json.JSONEncoder):
    def default(self, o):
        if isinstance(o, decimal.Decimal):
            if o % 1 > 0:
                return float(o)
            else:
                return int(o)
        return super(DecimalEncoder, self).default(o)

def adAuthFlow ():

	authority_url = (os.environ['authorityHostUrl'] + '/' +
					 os.environ['tenant'])
	RESOURCE = os.environ['resource']
	table = dynamodb.Table('smriotad-atDB')


	context = adal.AuthenticationContext(
		authority_url, validate_authority=os.environ['tenant'] != 'adfs',
		)

	token = context.acquire_token_with_client_credentials(
		RESOURCE,
		os.environ['clientId'],
		os.environ['clientSecret'])
		
	table.put_item(
		Item={
            'record_type' : 'accessToken',
            'value' : token,
            'timestamp' : time.time()
		}
	)

#	print('Here is the token:')
#	print(json.dumps(token, indent=2))

	return(token['accessToken'])

def check_room_reservation(roomname, temp_token):

	endpoint = os.environ['resource'] + '/' + 'v1.0' + '/users/' + roomname + '@smriotad.onmicrosoft.com/events'
	http_headers = {'Authorization': 'Bearer ' + temp_token,
					'Content-Type': 'application/json'}

	graph_data = requests.get(endpoint, headers=http_headers, stream=False)
	resp = graph_data.json()
	
	return resp

def release_room (roomname, temp_token):

	endpoint = sample_parameters['resource'] + '/' + 'v1.0' + '/users/conf.bh.bh.01.01.04.04.riffa@smriotad.onmicrosoft.com/events'
	http_headers = {'Authorization': 'Bearer ' + token['accessToken'],
					'Content-Type': 'application/json'}

	graph_data = requests.get(endpoint, headers=http_headers, stream=False)
	resp = graph_data.json()	

	print("\nThe graph_data URL for query event calendar is:%s"%resp)

	endpoint = sample_parameters['resource'] + '/' + 'v1.0' + '/users/adhoc.requester.bh.riffa@smriotad.onmicrosoft.com/events'
	http_body = {
				  "subject": "Ad-hoc meeting request - {15|30|45} minutes",
				  "start": {
					"dateTime": "2019-05-10T18:44:15.966Z",
					"timeZone": "UTC"
				  },
				  "end": {
					"dateTime": "2019-05-10T19:44:15.966Z",
					"timeZone": "UTC"
				  },
				  "attendees": [
					{
					  "emailAddress": {
						"address":"conf.bh.bh.01.01.04.04.riffa@smriotad.onmicrosoft.com",
						"name": "Conf BH BH Riffa"
					  },
					  "type": "resource"
					}
				  ]
				}

	graph_data = requests.post(endpoint, headers=http_headers, data=json.dumps(http_body), stream=False)
	resp = graph_data.json()	

	print("\nThe graph_data URL for add an new event calendar is:%s"%resp)
        
def lambda_handler(event, context):
    
	dynamodb = boto3.resource('dynamodb')
	state_table = dynamodb.Table(os.environ['DB_TABLE_STATE'])
	loc_table = dynamodb.Table(os.environ['DB_TABLE_LOCATION'])
	timer = event['occupancy_timer']
	
	list_rooms = loc_table.scan()
	room_results = []
	temp_token = adAuthFlow()

	for lrooms in list_rooms['Items']:
    
		roomid = os.environ['topic_state'] + "/" + lrooms['topic'][20:]

		resp_rooms = state_table.query(
				KeyConditionExpression=Key('topic').eq(roomid),
				ScanIndexForward=False,
				Limit=1
				)

		if not resp_rooms['Items']:
#			print("No room activity for ",lrooms['topic'][20:],", it is available")
			room_state = 1
		else:
			a_timestamp = time.time()
			for i in resp_rooms['Items']:
				t_difference_sec = (int(a_timestamp) - int(i['timestamp']) / 1000)
				room_state = t_difference_sec > timer

		if room_state:
			room_st = "Available"
			print("Room:",lrooms['topic'][20:], room_st)
		else:
			room_st = "Occupied"
			print("Room:",lrooms['topic'][20:], room_st)

		room_results.append({
					'Room' :   lrooms['topic'][20:],
					'State':   room_st
					})

		loc_table.update_item(
			Key={
				'topic': lrooms['topic'],
			},
			UpdateExpression='SET Occupancy = :val1',
			ExpressionAttributeValues={
				':val1': room_st
			}
		)
	return room_results

